
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <basic.h>
#include <bstdio.h>
#include <bstdlib.h>
#include <btron/vobj.h>
#include <btron/message.h>
#include <btron/file.h>

#include "xkobo.h"
#include "screen.h"
#include "manage.h"
#include "scenes.h"
#include "score.h"
#include "enemies.h"
#include "myship.h"
#include "radar.h"
#include "key.h"

#define GIGA             1000000000
#define BONUS_FIRSTTIME  2000
#define BONUS_EVERY      3000

extern bool cont_evloop;

_manage_state   _manage::state;
int             _manage::next_state_out;
int             _manage::next_state_next;
int             _manage::highscore = 0;
int             _manage::last_scene = 0;
int             _manage::scene_num;
int             _manage::ships;
int             _manage::level;
int             _manage::count;
int             _manage::score;
int             _manage::score_changed;
int             _manage::bonus_next;
int             _manage::flush_score_count = 0;
int             _manage::flush_ships_count = 0;
int             _manage::delay_count;
int             _manage::rest_cores;
int             _manage::return_value = 0;

static LINK* score_lnk = NULL;
static TC score_fname[] = {
  TK_x, TK_k, TK_o, TK_b, TK_o, 
  TK_PROD, 
  TK_s, TK_c, TK_o, TK_r, TK_e, 
  TNULL
};
extern M_EXECREQ* msg_exec;
extern LINK work_lnk;

struct ScoreFile {
    W highscore;
    W stage;
};

static void score_read(int *highscorep, int *stagep)
{
    W err, fd, remain;
    *highscorep = 0; *stagep = 0;
    if (score_lnk == NULL) {
        score_lnk = (LINK*)malloc(sizeof(LINK));
        memcpy(score_lnk, &work_lnk, sizeof(LINK));
        err = get_lnk(score_fname, score_lnk, F_BASED);
        if (err < 0) {
            free(score_lnk);
            score_lnk = NULL;
            return;
        }
    }

    fd = opn_fil(score_lnk, F_READ, NULL);
    if (fd < 0) {
        return;
    }

    err = fnd_rec(fd, F_TOPEND, 1<<31, 0, NULL);
    if (err < 0) {
        goto err_ret1;
    }

    ScoreFile sf;
    err = rea_rec(fd, 0, (B*)&sf, sizeof(ScoreFile), &remain, NULL);
    if (err < 0 || remain < sizeof(ScoreFile)) {
        goto err_ret1;
    }

    *highscorep = sf.highscore; *stagep = sf.stage;
    cls_fil(fd);
    return;

err_ret1:
    cls_fil(fd);
    return;
}

static void score_write(int highscore, int stage)
{
    W err = 0, fd = -1;
    if (score_lnk == NULL) {
        score_lnk = (LINK*)malloc(sizeof(LINK));
        memcpy(score_lnk, &work_lnk, sizeof(LINK));
        err = get_lnk(score_fname, score_lnk, F_BASED);
    }
    if (err >= 0) {
        fd = opn_fil(score_lnk, F_WRITE, NULL);
        if (fd >= 0) {
            goto label_succ;
        }
    }

    memcpy(score_lnk, &work_lnk, sizeof(LINK));
    fd = cre_fil(score_lnk, score_fname, NULL, 3, F_FIX);
    if (fd < 0) {
        free(score_lnk);
        score_lnk = NULL;
        return;
    }

label_succ:
    ScoreFile sf;
    sf.highscore = highscore;
    sf.stage = stage;

    err = fnd_rec(fd, F_TOPEND, 1<<31, 0, NULL);
    if (err >= 0) {
        err = wri_rec(fd, 0, (B*)&sf, sizeof(ScoreFile), NULL, NULL, 0);
        if (err < 0) {
            goto err_ret1;
        }
    }
    else {
        err = apd_rec(fd, (B*)&sf, sizeof(ScoreFile), 31, 0, 0);
        if (err < 0) {
            goto err_ret1;
        }
    }

    cls_fil(fd);
    return;

err_ret1:
    cls_fil(fd);
    del_fil(&work_lnk, score_lnk, 1);
    free(score_lnk);
    score_lnk = NULL;
    return;
}


void _manage::game_start()
{
    ships = SHIPS;
    level = 0;
    score = 0;
    flush_score_count = 0;
    bonus_next = BONUS_FIRSTTIME;
    screen.init_scene(scene_num);
    _manage::init_resources_to_play();
}

void _manage::next_scene()
{
    scene_num++;
    if (scene_num >= GIGA - 1) scene_num = GIGA - 2;
    screen.init_scene(scene_num);
}

void _manage::retry()
{
    if (!cheat_mode) ships--;
    if (ships <= 0){
        if (last_scene < scene_num)
            last_scene = scene_num;
        if (!cheat_mode){
            if (highscore < score) highscore = score;
            score_write(highscore, last_scene);
        }
        ships = 0;
        _manage::init_resources_title();
        return;
    }
}

void _manage::init_resources_title()
{
    state = title;
    screen.init_scene(-1);
    _manage::put_info();
}

void _manage::init_resources_to_play()
{
    state = playing;
    count = 0;
    delay_count = 0;
    flush_score_count = (flush_score_count) ? -1 : 0;
    flush_ships_count = 0;
    score_changed = 0;
    next_state_out = 0;
    next_state_next = 0;
    
    key.init();
    enemies.init();
    myship.init();
    rest_cores = screen.prepare();
    screen.generate_fixed_enemies();
    _manage::put_info();
    wchip.set_position(myship.get_virtx() << scale_log2,
                       myship.get_virty() << scale_log2);
}

void _manage::put_info()
{
    static char s[1024];
    int posx = MARGIN * 2 + (WSIZE << scale_log2) + 2;
    
    wbase.string(posx, 25, "SCORE");
    sprintf(s, "%09d", score);
    wbase.string(posx, 40, s);
    
    wbase.string(posx, 55, "STAGE");
    sprintf(s, "%09d", scene_num + 1);
    wbase.string(posx, 70, s);
    
    wbase.string(posx, 85, "SHIPS");
    if (!cheat_mode)
        sprintf(s, "%09d", ships);
    else
        sprintf(s, "999999999");
    wbase.string(posx, 100, s);
}

void _manage::put_score()
{
    static char s[1024];
    int posx = MARGIN * 2 + (WSIZE << scale_log2) + 2;
    
    sprintf(s, "%09d", score);
    wbase.string(posx, 40, s);
    score_changed = 0;
}

void _manage::flush_score()
{
    static char s[1024];
    int posx = MARGIN * 2 + (WSIZE << scale_log2) + 2;

    flush_score_count--;
    if (flush_score_count & 1) return;
    if (flush_score_count & 2){
        wbase.string(posx, 25, "            ");
        wbase.string(posx, 40, "                  ");
    }
    else {
        wbase.string(posx, 25, "SCORE");
        sprintf(s, "%09d", score);
        wbase.string(posx, 40, s);
    }
    if (flush_score_count == 0) flush_score_count = -1;
}

void _manage::flush_ships()
{
    static char s[1024];
    int posx = MARGIN * 2 + (WSIZE << scale_log2) + 2;

    flush_ships_count--;
    if (flush_ships_count & 1) return;
    if (flush_ships_count & 2){
        wbase.string(posx, 85, "          ");
        wbase.string(posx, 100, "                  ");
    }
    else {
        wbase.string(posx, 85, "SHIPS");
        sprintf(s, "%09d", ships);
        wbase.string(posx, 100, s);
    }
}

static void redisp()
{
    wbase.expose_backing();
    wchip.expose_backing();
    wradar.expose_backing();
}

static void on_redisp()
{
    do {
        if (wsta_dsp(win::xevt.s.wid, NULL, NULL) == 0) {
            break;
        }
        redisp();
    } while (wend_dsp(win::xevt.s.wid) > 0);
}

static int ptr_sts = 1;

static void update_pointer_status()
{
  if (ptr_sts) {
    if (win::xevt.s.wid == wbase.w && win::xevt.s.cmd == W_WORK) {
      gdsp_ptr(0);
      ptr_sts = 0;
    }
  }
  else {
    if (win::xevt.s.wid != wbase.w || win::xevt.s.cmd != W_WORK) {
      gdsp_ptr(-1);
      ptr_sts = 1;
    }
  }
}

void _manage::eventloop()
{
    W clt, err;
    RECT rsz;

    for (;;) {
        wget_evt(&win::xevt, NOWAIT);

        switch (win::xevt.s.type) {
        case EV_NULL:
            update_pointer_status();
            /* FALLTHROUGH */
        case EV_NOMSG:
            //          on_idle(&wev0, inst);
            // TODO: ǡޥκɸѤäƤʤǧ뤳
            return;
        case EV_REQUEST:
            switch (win::xevt.g.cmd) {
            case W_REDISP:
                // TODO: ŬʤΤǡľ
                on_redisp();
                break;
            case W_PASTE:
                wrsp_evt(&win::xevt, 1); /* NACK */
                break;
            case W_DELETE:
            case W_FINISH:
                wrsp_evt(&win::xevt, 0); /* ACK */
                cont_evloop = false;
                break;
            }
            break;
        case EV_RSWITCH:
            on_redisp();
            /* FALLTHROUGH */
        case EV_SWITCH:
            key.clear();
            /* FALLTHROUGH */
        case EV_BUTDWN:
            update_pointer_status();
            switch (win::xevt.s.cmd) {
            case W_PICT:
              clt = wchk_dck(win::xevt.s.time);
              if (clt == W_DCLICK){
                cont_evloop = false;
                break;
              }
              else if (clt == W_CLICK){
                break; /* no-operation */
              }
              /* FALLTHROUGH */
            case W_LTHD:
            case W_LBHD:
            case W_RTHD:
            case W_RBHD:
            case W_TITL:
            case W_FRAM:
              clt = wchk_dck(win::xevt.s.time);
              if (clt == W_PRESS || clt == W_QPRESS){
                err = wmov_drg(&win::xevt, NULL);
                if (err >= 1){
                  on_redisp();
                }
              }
              break;
            case W_WORK:
            default:
              key.mouse_press(1);
              break;
            }
            break;
        case EV_BUTUP:
            update_pointer_status();
            key.mouse_release(1);
            break;
        case EV_INACT:
            key.clear();
            break;
        case EV_DEVICE:
            oprc_dev(&(win::xevt.e), NULL, 0);
            break;
        case EV_MSG:
            clr_msg(MM_ALL, MM_ALL);
            break;
        case EV_MENU:
            update_pointer_status();
            break;
        case EV_AUTKEY:
            break;
        case EV_KEYDWN:
            if (win::xevt.s.stat & ES_CMD) {
              switch (win::xevt.e.data.key.code) {
              case TK_e:
              case TK_E:
                key.press(TK_q);
                break;
              case TK_w:
              case TK_W:
                wexe_dmn(1);
                break;
              default:
                break;
              }
            }
            else {
              key.press(win::xevt.e.data.key.code);
            }
            break;
        case EV_KEYUP:
            key.release(win::xevt.e.data.key.code);
            break;
        }
    }
}

/****************************************************************************/
void _manage::init()
{
    score_read(&highscore, &last_scene);
    count = 0;
    ships = 0;
    level = 0;
    scene_num = last_scene;
    flush_ships_count = 0;
    delay_count = 0;
    screen.init();
    _manage::init_resources_title();
}

int _manage::mainloop()
{
    _manage::eventloop();
    
    if (state != playing){
        return return_value;
    }
    
    count++;
    if (delay_count) delay_count--;
    
    if (delay_count == 1){
        if (enemies.exist_pipe()){
            delay_count = 2;
        }
        else {
            _manage::put_info();
            if (next_state_out){
                _manage::retry();
                if (ships <= 0){
                    return return_value;
                }
            }
            if (next_state_next){
                _manage::next_scene();
            }
            _manage::init_resources_to_play();
            return return_value;
        }
    }
    
    myship.move();
    enemies.move();
    myship.hit_structure();
    
    myship.put();
    enemies.put();
    if (score_changed) _manage::put_score();
    if (flush_score_count > 0) _manage::flush_score();
    if (flush_ships_count) _manage::flush_ships();
    radar.trace_myship();
    wchip.set_position(myship.get_virtx() << scale_log2,
                       myship.get_virty() << scale_log2);
    
    return return_value;
}

void _manage::lost_myship()
{
    if (delay_count == 0)
        delay_count = 20;
    next_state_out = 1;
}

void _manage::destroyed_a_core()
{
    rest_cores--;
    if (rest_cores == 0){
        next_state_next = 1;
        delay_count = 50;
    }
    screen.generate_fixed_enemies();
}

void _manage::start_key()
{
    switch (state){
      case title:{
          _manage::game_start();
          break;
      }
      case paused:{
          state = playing;
          break;
      }
      case playing:{
          state = paused;
          wchip.back();
          break;
      }
    }
}

void _manage::exit_key()
{
    if (state == title){
        return_value = 1;
    }
    ships = 0;
    _manage::retry();
    _manage::put_info();
}

void _manage::plus_key()
{
    if (state == title){
        if ((scene_num < last_scene) || cheat_mode)
            scene_num++;
        _manage::put_info();
    }
}

void _manage::minus_key()
{
    if (state == title){
        scene_num--;
        if (scene_num <= 0) scene_num = 0;
        _manage::put_info();
    }
}

void _manage::add_score(int sc)
{
    score += sc;
    if (score >= GIGA) score = GIGA - 1;
    else if (!cheat_mode){
        if (score >= bonus_next){
            bonus_next += BONUS_EVERY;
            ships++;
            flush_ships_count = 50;
        }
        if ((score >= highscore) && (flush_score_count == 0)){
            flush_score_count = 50;
        }
    }
    score_changed = 1;
}

void _manage::get_highscore(char *s)
{
    sprintf(s, "high score : %09d", highscore);
}    
