
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <basic.h>
#include <bstdio.h>
#include <bstdlib.h>
#include <btron/dp.h>
#include <btron/hmi.h>
#include <btron/libapp.h>

#include "xkobo.h"
#include "config.h"
#include "tiff.h"
#include "screen.h"
#include "manage.h"
#include "score.h"
#include "key.h"
#include "random.h"
#include "version.h"

CSPEC           cspec_system;

win_cmap        wbase;
win_chip        wchip;
win_backing     wradar;
int             scale_log2;
int             mouse_x = -1;
int             mouse_y = -1;
int             cheat_mode;

// SSHelvetica
FSSPEC font_normal = {
  {TK_S, TK_S, TK_H, TK_e, TK_l, TK_v, TK_e, TK_t, TK_i, TK_c, TK_a, TNULL},
  0x000000c0, // gothic
  FT_REGULAR|FT_BOLD|FT_RIGHT|FT_SOLID|FT_HDRAW|FT_PROP|FT_GRAYSCALE,
  {12, 12},
};
FSSPEC font_title = {
  {TK_S, TK_S, TK_T, TK_i, TK_m, TK_e, TK_s, TNULL},
  FTC_MINCHO,
  FT_REGULAR|FT_BOLD|FT_RIGHT|FT_SOLID|FT_HDRAW|FT_PROP|FT_GRAYSCALE,
  {24, 24},
};

static W        main_tid;

static int      signal_delivered = 1;

bool            cont_evloop;

M_EXECREQ*      msg_exec = NULL;
static TC       work_fpath[] = {TC_FDLM, TK_S, TK_Y, TK_S, TNULL};
LINK            work_lnk;

BMP* XCreatePixmap(int w, int h)
{
    BMP* p = (BMP*)malloc(sizeof(BMP) + sizeof(UB*) * 28);
    p->planes = SCREEN.planes;
    p->pixbits = SCREEN.pixbits;
    p->bounds.c.left = 0;
    p->bounds.c.top = 0;
    p->bounds.c.right = w;
    p->bounds.c.bottom = h;
    bmalloc(p);
    return p;
}

void motion_callback(win&)
{
    mouse_x = win::xevt.s.pos.x;
    mouse_y = win::xevt.s.pos.y;
    mouse_x -= (WSIZE>>(1-scale_log2));
    mouse_y -= (WSIZE>>(1-scale_log2));
    if ((mouse_x == -1) && (mouse_y == -1)) return;
    key.mouse_position(mouse_x, mouse_y);
}

void leave_callback(win&)
{
    mouse_x = -1;
    mouse_y = -1;
}

/*
void client_message_callback(win&)
{
    Atom wm_protocols, wm_delete_window;
    wm_protocols = XInternAtom(win::disp, "WM_PROTOCOLS", False);
    wm_delete_window = XInternAtom(win::disp, "WM_DELETE_WINDOW", False);
    if ((win::xevt.xclient.message_type == wm_protocols) &&
        (win::xevt.xclient.data.l[0] == wm_delete_window)){
        _manage::exit_key();
        _manage::exit_key();
    }
}
*/

void put_usage()
{
    printf("\nXKOBO %s\n", XKOBO_VERSION);
    printf("Usage: xkobo -hiscores\n");
    printf("       xkobo [-doublesize]\n");
    printf("             [-fast]\n");
    printf("             [-fit]\n");
    printf("             [-cheat]\n");
    printf("             [-display]\n");
    printf("             [-wait XXX]\n");
    printf("\n");
    exit(1);
}

static void itimer_task(W arg)
{
    for (;;) {
        dly_tsk(arg);
        signal_delivered = 1;
        wup_tsk(main_tid);
    }

    ext_tsk();
}

xkobo_fusen fusen;

/*           xkobo          */

int main_xkobo(int argc, char *argv[])
{
    int sprite_policy = 1;
    int cmap_policy = 1;
    int wait_msec = WAIT_MSEC;
    int i;
    xkobo_fusen oldfusen;
    WDSTAT stat;
    W err;

    err = get_fsn_data(msg_exec, &fusen, sizeof(xkobo_fusen));
    if (err < 0 || fusen.dlen != sizeof(xkobo_fusen) - sizeof(UH)) {
      fusen.dlen = sizeof(xkobo_fusen) - sizeof(UH);
      fusen.padding1 = 0;
      fusen.base_x = 50;
      fusen.base_y = 50;
      fusen.score_x = 480;
      fusen.score_y = 40;
    }
    memcpy(&oldfusen, &fusen, sizeof(xkobo_fusen));
    
    scale_log2 = 0;
    cheat_mode = 0;
    for (i=1; i<argc; i++){
        if (!strcmp("-speed", argv[i]))
            sprite_policy = 0;
        else if (!strcmp("-fast", argv[i]))
            sprite_policy = 0;
        else if (!strcmp("-doublesize", argv[i]))
            scale_log2 = 1;
        else if (!strcmp("-private", argv[i]))
            cmap_policy = 0;
        else if (!strcmp("-fit", argv[i]))
            cmap_policy = 0;
        else if (!strcmp("-cheat", argv[i]))
            cheat_mode = 1;
        else if (!strcmp("-display", argv[i]))
            win::set_disp_string(argv[++i]);
        else if (!strcmp("-wait", argv[i]))
            wait_msec = atoi(argv[++i]);
        else put_usage();
    }

    get_lnk(work_fpath, &work_lnk, F_NORM);

    chg_emk(EM_ALL);

    dopn_dat(NULL);
    getscreen();
    GetStdCSPEC(&cspec_system, SCREEN.planes, SCREEN.pixbits);

    //    wchip .event(MotionNotify,  &motion_callback        );
    //    wchip .event(LeaveNotify,   &leave_callback         );
    
    wbase.make(NULL, 0, 0,
               MARGIN * 3 + ((WSIZE + MAP_SIZEX) << scale_log2),
               MARGIN * 2 + (WSIZE << scale_log2), cmap_policy);
    wbase.setwopt(fusen.base_x, fusen.base_y, WA_WORK);
    wchip.make(&wbase, MARGIN, MARGIN,
               WSIZE << scale_log2, WSIZE << scale_log2,
               (SCREEN_SIZEX + WSIZE) << scale_log2,
               (SCREEN_SIZEY + WSIZE) << scale_log2,
               320 << scale_log2, 240 << scale_log2, sprite_policy);
    wradar.make(&wbase,
                MARGIN * 2 + (WSIZE << scale_log2), 
                MARGIN + ((WSIZE - MAP_SIZEY) << scale_log2),
                MAP_SIZEX << scale_log2, MAP_SIZEY << scale_log2);
    
    if (link_tiff(spdata, wbase, wchip)) return 1;
    
//    wbase.title("xkobo");
    wbase.setparentwid(msg_exec->pwid);
    wbase.font(&font_normal);
    wbase.foreground(0x10ffffff);
    wbase.background(0x10000000);
    wbase.map();
    wbase.clear();
    wchip.foreground(0x10ffffff);
    wchip.background(0x10000000);
    wchip.setborder(0x10000000);
    wchip.erase_cursor();
    wchip.map();
    wradar.foreground(0x10ffffff);
    wradar.background(0x10000000);
    wradar.setborder(0x10000000);
    wradar.map();

    osta_prc(msg_exec->vid, wbase.getwid());
    
    rand_num.init();
    manage.init();
    
    main_tid = get_tid();
    W itimer = cre_tsk(itimer_task, 192, wait_msec);

    cont_evloop = 1;
    for (;;){
        if (wait_msec && (signal_delivered == 0)) slp_tsk(-1);
        signal_delivered = 0;
        can_wup(0); // ٱ˥塼󥰤줿Τ
        if (manage.mainloop()) break;
        if (!cont_evloop) break;
    }

    ter_tsk(itimer);

    stat.attr = WA_WORK;
    wget_sts(wbase.w, &stat, NULL);
    fusen.base_x = stat.r.c.left;
    fusen.base_y = stat.r.c.top;

    put_fsn_data(msg_exec, &fusen, &oldfusen, True);
    wbase.unmap();

    gdsp_ptr(-1);
    
    return 0;
}

extern "C" W MAIN(MESSAGE *msg)
{
    switch (msg->msg_type) {
    case DISPREQ:
        oend_req(((M_DISPREQ*)msg)->vid, -1);
        ext_prc(1);
    case PASTEREQ:
        oend_req(((M_PASTEREQ*)msg)->vid, -1);
        ext_prc(1);
    case EXECREQ:
        if ((((M_EXECREQ*)msg)->mode & 2) == 0) {
            /* non-komono execution */
            oend_prc(((M_EXECREQ*)msg)->vid, NULL, 0);
            ext_prc(1);
        }
        break;
    default:
        /* CLI exec, etc/ */
        ext_prc(1);
    }
    msg_exec = (M_EXECREQ*)msg;

    char* argv[] = {"xkobo",/* "-cheat",*/ /*"-doublesize",*/ NULL};
    W ret = main_xkobo(sizeof(argv)/sizeof(char*)-1, argv);

    ext_prc(ret);
}
