
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <basic.h>
#include <bstdio.h>
#include <bstdlib.h>
#include <btron/libapp.h>
#include "xevent.h"
#include "xlbacking.h"

//--------------------------------------------------------------------------//
//                     class     win_backing                                //
//--------------------------------------------------------------------------//

extern BMP* XCreatePixmap(int w, int h);
extern CSPEC cspec_system;

win_backing::win_backing()
{
    mask |= Expose;
}

win_backing::~win_backing()
{
    if (p) {
      bfree(p);
      free(p);
    }
}

void win_backing::make(win *back,int wx,int wy,int sizex,int sizey)
{
    this->win::make(back,wx,wy,sizex,sizey);
    p = XCreatePixmap(sx, sy);
    
    p_gc = gopn_mem(NULL, p, (B*)&cspec_system);

    fgcolor = 0x10ffffff;
    bgcolor = 0x10000000;

    RECT r;
    r.c.left = 0; r.c.top = 0; r.c.right = sx; r.c.bottom = sy;
    bgpat.spat.kind = 0;
    bgpat.spat.hsize = bgpat.spat.vsize = 1;
    bgpat.spat.fgcol = fgcolor;
    bgpat.spat.bgcol = bgcolor;
    bgpat.spat.mask = FILL100;
    gfil_rec(p_gc, r, &bgpat, 0, G_STORE);
}

void win_backing::map()
{
    this->win::map();
    w_gc = wget_gid(w);
}

void win_backing::unmap()
{
    this->win::unmap();
    gcls_env(p_gc);
}

void win_backing::expose_backing()
{
    back(); // TODO: This is not at all optimal
}

void win_backing::clear()
{
    RECT r; r.c.left = 0; r.c.right = sx; r.c.top = 0; r.c.bottom = sy;
    PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
    pat.spat.fgcol = pat.spat.bgcol = bgcolor; 
    pat.spat.mask = FILL100;
    gfil_rec(p_gc, r, &pat, 0, G_STORE);
    r.c.left += this->x; r.c.right += this->x;
    r.c.top += this->y; r.c.bottom += this->y;
    gfil_rec(w_gc, r, &pat, 0, G_STORE);
}

void win_backing::clear(int x,int y,int h,int v)
{
    RECT r; r.c.left = x; r.c.right = x+h; r.c.top = y; r.c.bottom = y+v;
    PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
    pat.spat.fgcol = pat.spat.bgcol = bgcolor; 
    pat.spat.mask = FILL100;
    gfil_rec(p_gc, r, &pat, 0, G_STORE);
    r.c.left += this->x; r.c.right += this->x;
    r.c.top += this->y; r.c.bottom += this->y;
    gfil_rec(w_gc, r, &pat, 0, G_STORE);
}

void win_backing::foreground(COLOR i)
{
    fgcolor = i;
}

void win_backing::background(COLOR i)
{
    bgcolor = i;
}

void win_backing::font(FSSPEC *f)
{
    fntp = f;
}

