
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#ifndef XKOBO_H_XLBACKING
#define XKOBO_H_XLBACKING

#include <basic.h>
#include <btron/dp.h>
#include <btron/hmi.h>
#include <bstdio.h>
#include <tstring.h>
#include "xlwin.h"

/*
 *
 *     win  ---- win_backing ----
 *                            \
 *                             \
 *
 */

class win_backing : public win{

  protected:
    GID w_gc;
    COLOR fgcolor;
    COLOR bgcolor;
    BMP* p;
    GID p_gc;
    FSSPEC *fntp;
    
  public:
    win_backing();
    virtual ~win_backing();
    void make(win *back,int wx=0,int wy=0,int sizex=200,int sizey=200);
    void map();
    void unmap();
    void clear();
    void clear(int x,int y,int h,int v);
    virtual void expose_backing();                 /* from eventroop() */
    void foreground(COLOR c);
    void background(COLOR c);
    void font(FSSPEC *f);
    inline void point_back(int x,int y){
      PNT pt; pt.x = x; pt.y = y;
      gdra_pnt(p_gc, pt, fgcolor, G_STORE);
    }
    inline void point(int x,int y){
      PNT pt; pt.x = x; pt.y = y;
      gdra_pnt(p_gc, pt, fgcolor, G_STORE);
      pt.x += this->x; pt.y += this->y;
      gdra_pnt(w_gc, pt, fgcolor, G_STORE);
    }
    inline void string_back(int x,int y,char *t){
      TC buf[256]; eucstotcs(buf, t);
      gset_fon(p_gc, fntp);
      gset_chc(p_gc, fgcolor, bgcolor);
      gdra_stp(p_gc, x, y, buf, sizeof(buf)/sizeof(TC), G_STORE);
    }
    inline void string(int x,int y,char *t){
      TC buf[256]; eucstotcs(buf, t);
      gset_fon(p_gc, fntp);
      gset_chc(p_gc, fgcolor, bgcolor);
      gdra_stp(p_gc, x, y, buf, sizeof(buf)/sizeof(TC), G_STORE);
      gset_fon(w_gc, fntp);
      gset_chc(w_gc, fgcolor, bgcolor);
      gdra_stp(w_gc, x+this->x, y+this->y, 
               buf, sizeof(buf)/sizeof(TC), G_STORE);
    }
    inline void rectangle_back(int x,int y,int h,int v){
      RECT r; r.c.left = x; r.c.right = x+h; r.c.top = y; r.c.bottom = y+v;
      PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
      pat.spat.fgcol = pat.spat.bgcol = fgcolor; 
      pat.spat.mask = FILL100;
      gfra_rec(p_gc, r, ((LN_SOLID << 8) | 1), &pat, 0, G_STORE);
    }
    inline void rectangle(int x,int y,int h,int v){
      RECT r; r.c.left = x; r.c.right = x+h; r.c.top = y; r.c.bottom = y+v;
      PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
      pat.spat.fgcol = pat.spat.bgcol = fgcolor; 
      pat.spat.mask = FILL100;
      gfra_rec(p_gc, r, ((LN_SOLID << 8) | 1), &pat, 0, G_STORE);
      r.c.left += this->x; r.c.right += this->x;
      r.c.top += this->y; r.c.bottom += this->y;
      gfra_rec(w_gc, r, ((LN_SOLID << 8) | 1), &pat, 0, G_STORE);
    }
    inline void fillrectangle_back(int x,int y,int h,int v){
      RECT r; r.c.left = x; r.c.right = x+h; r.c.top = y; r.c.bottom = y+v;
      PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
      pat.spat.fgcol = pat.spat.bgcol = fgcolor; 
      pat.spat.mask = FILL100;
      gfil_rec(p_gc, r, &pat, 0, G_STORE);
    }
    inline void fillrectangle(int x,int y,int h,int v){
      RECT r; r.c.left = x; r.c.right = x+h; r.c.top = y; r.c.bottom = y+v;
      PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
      pat.spat.fgcol = pat.spat.bgcol = fgcolor; 
      pat.spat.mask = FILL100;
      gfil_rec(p_gc, r, &pat, 0, G_STORE);
      r.c.left += this->x; r.c.right += this->x;
      r.c.top += this->y; r.c.bottom += this->y;
      gfil_rec(w_gc, r, &pat, 0, G_STORE);
    }
    inline void back(){
      RECT r1; r1.c.left = r1.c.top = 0; r1.c.right = sx; r1.c.bottom = sy;
      RECT r2;
      r2.c.left = this->x + r1.c.left; r2.c.right = this->x + r1.c.right;
      r2.c.top = this->y + r1.c.top; r2.c.bottom = this->y + r1.c.bottom;
      W err = gcop_bmp(p_gc, &r1, w_gc, &r2, NULL, G_STORE);
    }
    inline void back(int x,int y,int h,int v){
      RECT r1; r1.c.left = x; r1.c.top = y; r1.c.right = x+h; r1.c.bottom = y+v;
      RECT r2;
      r2.c.left = this->x + r1.c.left; r2.c.right = this->x + r1.c.right;
      r2.c.top = this->y + r1.c.top; r2.c.bottom = this->y + r1.c.bottom;
      gcop_bmp(p_gc, &r1, w_gc, &r2, NULL, G_STORE);
    }
};

#endif // XKOBO_H_XLBACKING
