
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <basic.h>
#include <bstdlib.h>
#include <btron/hmi.h>
#include <btron/dp.h>
#include <btron/libapp.h>
#include "xlchip.h"

extern BMP* XCreatePixmap(int w, int h);
extern CSPEC cspec_system;
     
win_chip::win_chip()
{
    spr_max = sprite;
}

win_chip::~win_chip()
{
    if (chip_mask) {
      bfree(chip_mask);
      free(chip_mask);
    }
    if (chip) {
      bfree(chip);
      free(chip);
    }
    if (p_store) {
      bfree(p_store);
      free(p_store);
    }
} 

void win_chip::make(win *back,int wx,int wy, int sizex,int sizey,
                    int vsizex, int vsizey, int csizex, int csizey, int policy)
{
    this->win_scroll::make(back,wx,wy,sizex,sizey,vsizex,vsizey);
    csx = csizex;
    csy = csizey;
    store_policy = policy;   /* 0 : speed   1 : memory */
    if (store_policy == 0){
        p_store = XCreatePixmap(vsx,vsy);
    }
    else {
        p_store = XCreatePixmap(sizex,sizey);
    }
    chip      = XCreatePixmap(csx,csy);
    chip_mask = XCreatePixmap(csx,csy);

    p_store_gc = gopn_mem(NULL, p_store, (B*)&cspec_system);
    chip_gc = gopn_mem(NULL, chip, (B*)&cspec_system);
    chip_mask_gc = gopn_mem(NULL, chip_mask, (B*)&cspec_system);
}

void win_chip::map()
{
    this->win_scroll::map();
}

void win_chip::unmap()
{
    this->win_scroll::unmap();
    gcls_env(p_store_gc);
    gcls_env(chip_gc);
    gcls_env(chip_mask_gc);
}

void win_chip::torus_copy_from_chip_and_store(int x, int y, int h, int v,
                                              int x1, int x2)
{
    this->copy_from_chip_and_store(x,y,h,v,x1,x2);
    if (x1 < sx){
        this->copy_from_chip_and_store(x,y,h,v,x1+vsx-sx,x2);
        if (x2 < sy){
            this->copy_from_chip_and_store(x,y,h,v,x1,x2+vsy-sy);
            this->copy_from_chip_and_store(x,y,h,v,x1+vsx-sx,x2+vsy-sy);
        }
    }
    else if (x2 < sy){
        this->copy_from_chip_and_store(x,y,h,v,x1,x2+vsy-sy);
    }
}

void win_chip::copy_from_chip_and_store(int x, int y, int h, int v,
                                              int x1, int x2)
{
    RECT r1, r2;
    r1.c.left = x; r1.c.top = y;
    r1.c.right = x+h; r1.c.bottom = y+v;
    r2.c.left = x1; r2.c.top = x2;
    r2.c.right = x1+h; r2.c.bottom = x2+v;

    gcop_bmp(chip_gc, &r1, p_gc, &r2, NULL, G_STORE);
    if (store_policy == 0){
        gcop_bmp(chip_gc, &r1, p_store_gc, &r2, NULL, G_STORE);
    }
}

int win_chip::clip(int& cx, int& cy, int& x, int& y, int& h, int& v)
{
    int x2 = x + h;
    int y2 = y + h;
    if (x < vx){
        cx += (vx - x);
        x = vx;
    }
    if (y < vy){
        cy += (vy - y);
        y = vy;
    }
    if (x2 > vx + sx) x2 = vx + sx;
    if (y2 > vy + sy) y2 = vy + sy;
    h = x2 - x;
    v = y2 - y;
    if ((h < 0) || (v < 0)){
        h = 0;
        v = 0;
        return 1;
    }
    return 0;
}

void win_chip::store()
{
    if (store_policy == 0) {
        RECT r1;
        r1.c.left = 0; r1.c.top = 0;
        r1.c.right = vsx; r1.c.bottom = vsy;
        gcop_bmp(p_gc, &r1, p_store_gc, NULL, NULL, G_STORE);
    }
}

void win_chip::set_position(int vposx, int vposy)
{
    RECT r1, r2;
    vx = vposx;
    vy = vposy;
    _sprite *spp;

    if (store_policy == 0){
        /*  policy : speed */
        for (spp=sprite; spp<spr_max; spp++){
            r1.c.left = spp->cx; r1.c.top = spp->cy;
            r1.c.right = spp->cx + spp->h; r1.c.bottom = spp->cy + spp->v;
            r2.c.left = spp->x; r2.c.top = spp->y;
            r2.c.right = spp->x + spp->h; r2.c.bottom = spp->y + spp->v;
            gcop_bmp(chip_mask_gc, &r1, p_gc, &r2, NULL, G_AND);
            gcop_bmp(chip_gc,      &r1, p_gc, &r2, NULL, G_OR);
        }
        r1.c.left = vx; r1.c.top = vy;
        r1.c.right = vx+sx; r1.c.bottom = vy+sy;
        r2.c.left = this->x; r2.c.top = this->y;
        r2.c.right = this->x + sx; r2.c.bottom = this->y + sy;
        gcop_bmp(p_gc, &r1, w_gc, &r2, NULL, G_STORE);
        for (spp=sprite; spp<spr_max; spp++){
            r1.c.left = spp->x; r1.c.top = spp->y;
            r1.c.right = spp->x + spp->h; r1.c.bottom = spp->y + spp->v;
            gcop_bmp(p_store_gc, &r1, p_gc, NULL, NULL, G_STORE);
        }
        spr_max = sprite;
    }
    else {
        /*  policy : memory */
        r1.c.left = vx; r1.c.top = vy;
        r1.c.right = vx+sx; r1.c.bottom = vy+sy;
        r2.c.left = 0; r2.c.top = 0;
        r2.c.right = sx; r2.c.bottom = sy;
        gcop_bmp(p_gc, &r1, p_store_gc, &r2, NULL, G_STORE);
        for (spp=sprite; spp<spr_max; spp++){
            if (!(this->clip(spp->cx, spp->cy, spp->x, spp->y,
                             spp->h, spp->v))){
                r1.c.left = spp->cx; r1.c.top = spp->cy;
                r1.c.right = spp->cx+spp->h; r1.c.bottom = spp->cy+spp->v;
                r2.c.left = spp->x; r2.c.top = spp->y;
                r2.c.right = spp->x+spp->h; r2.c.bottom = spp->y+spp->v;
                gcop_bmp(chip_mask_gc, &r1, p_gc, &r2, NULL, G_AND);
                gcop_bmp(chip_gc,      &r1, p_gc, &r2, NULL, G_OR);
            }
        }
        r1.c.left = vx; r1.c.top = vy;
        r1.c.right = vx+sx; r1.c.bottom = vy+sy;
        r2.c.left = this->x; r2.c.top = this->y;
        r2.c.right = this->x + sx; r2.c.bottom = this->y + sy;
        gcop_bmp(p_gc, &r1, w_gc, &r2, NULL, G_STORE);
        for (spp=sprite; spp<spr_max; spp++){
            r1.c.left = spp->x - vx; r1.c.top = spp->y - vy;
            r1.c.right = r1.c.left + spp->h;
            r1.c.bottom = r1.c.top + spp->v;
            r2.c.left = spp->x; r2.c.top = spp->y;
            r2.c.right = r2.c.left + spp->h;
            r2.c.bottom = r2.c.top + spp->v;
            gcop_bmp(p_store_gc, &r1, p_gc, &r2, NULL, G_STORE);
        }
        spr_max = sprite;
    }
}
