
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include "xlscroll.h"

//--------------------------------------------------------------------------//
//                     class     win_scroll                                 //
//--------------------------------------------------------------------------//

extern BMP* XCreatePixmap(int w, int h);
extern CSPEC cspec_system;

win_scroll::win_scroll()
{
    ;
}

win_scroll::~win_scroll()
{
    ;
}

void win_scroll::make(win *back,int wx,int wy, int sizex,int sizey,
                      int vsizex, int vsizey)
{
    vsx = vsizex;
    vsy = vsizey;
    vx  = 0;
    vy  = 0;
    this->win::make(back,wx,wy,sizex,sizey);
    p = XCreatePixmap(vsx, vsy);
    
    p_gc = gopn_mem(NULL, p, (B*)&cspec_system);

    fgcolor = 0x10ffffff;
    bgcolor = 0x10000000;

    RECT r;
    r.c.left = 0; r.c.top = 0; r.c.right = vsx; r.c.bottom = vsy;
    bgpat.spat.kind = 0;
    bgpat.spat.hsize = bgpat.spat.vsize = 1;
    bgpat.spat.fgcol = fgcolor;
    bgpat.spat.bgcol = bgcolor;
    bgpat.spat.mask = FILL100;
    gfil_rec(p_gc, r, &bgpat, 0, G_STORE);
}

void win_scroll::clear()
{
    RECT r; r.c.left = 0; r.c.right = vsx; r.c.top = 0; r.c.bottom = vsy;
    PAT pat; pat.spat.kind = 0; pat.spat.hsize = pat.spat.vsize = 1;
    pat.spat.fgcol = pat.spat.bgcol = bgcolor; 
    pat.spat.mask = FILL100;
    gfil_rec(p_gc, r, &pat, 0, G_STORE);
    r.c.left += this->x; r.c.right += this->x;
    r.c.top += this->y; r.c.bottom += this->y;
    gfil_rec(w_gc, r, &pat, 0, G_STORE);
}

void win_scroll::expose_backing()
{
    // TODO: This is not at all optimal
    RECT r1;
    r1.c.left = vx; r1.c.top = vy; 
    r1.c.right = vx+sx; r1.c.bottom = vy+sy;
    RECT r2;
    r2.c.left = this->x; r2.c.right = this->x + sx;
    r2.c.top = this->y; r2.c.bottom = this->y + sy;
    W err = gcop_bmp(p_gc, &r1, w_gc, &r2, NULL, G_STORE);
}

void win_scroll::set_position(int vposx, int vposy)
{
    vx = vposx;
    vy = vposy;

    RECT r1;
    r1.c.left = vx; r1.c.top = vy; 
    r1.c.right = vx+sx; r1.c.bottom = vy+sy;
    RECT r2;
    r2.c.left = this->x; r2.c.right = this->x + sx;
    r2.c.top = this->y; r2.c.bottom = this->y + sy;
    gcop_bmp(p_gc, &r1, w_gc, &r2, NULL, G_STORE);
}

void win_scroll::torus()
{
    int x = vsx - sx;
    int y = vsy - sy;

    RECT r1, r2;
    r1.c.left = 0; r1.c.top = 0; 
    r1.c.right = sx; r1.c.bottom = y;
    r2.c.left = x; r2.c.top = 0;
    r2.c.right = x + sx; r2.c.bottom = y;
    gcop_bmp(p_gc, &r1, p_gc, &r2, NULL, G_STORE);

    r1.c.left = 0; r1.c.top = 0; 
    r1.c.right = vsx; r1.c.bottom = sy;
    r2.c.left = 0; r2.c.top = y;
    r2.c.right = vsx; r2.c.bottom = y + sy;
    gcop_bmp(p_gc, &r1, p_gc, &r2, NULL, G_STORE);
}
